package org.eclipse.jdt.internal.compiler.flow;

package .tmp;

import org.eclipse.jdt.internal.compiler.ast.IfStatement;
import org.eclipse.jdt.internal.compiler.flow.ConditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.flow.UnconditionalFlowInfo;
import org.eclipse.jdt.internal.compiler.lookup.FieldBinding;
import org.eclipse.jdt.internal.compiler.lookup.LocalVariableBinding;

public abstract class FlowInfo {
  public int tagBits;
  
  public static final int REACHABLE = 0;
  
  public static final int UNREACHABLE_OR_DEAD = 1;
  
  public static final int UNREACHABLE_BY_NULLANALYSIS = 2;
  
  public static final int UNREACHABLE = 3;
  
  public static final int NULL_FLAG_MASK = 4;
  
  public static final int UNKNOWN = 1;
  
  public static final int NULL = 2;
  
  public static final int NON_NULL = 4;
  
  public static final int POTENTIALLY_UNKNOWN = 8;
  
  public static final int POTENTIALLY_NULL = 16;
  
  public static final int POTENTIALLY_NON_NULL = 32;
  
  public static final int UNROOTED = 64;
  
  public static final int FREE_TYPEVARIABLE = 48;
  
  public static final UnconditionalFlowInfo DEAD_END = new UnconditionalFlowInfo();
  
  static {
    DEAD_END.tagBits = 3;
  }
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo addInitializationsFrom(org.eclipse.jdt.internal.compiler.flow.FlowInfo paramFlowInfo);
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo addNullInfoFrom(org.eclipse.jdt.internal.compiler.flow.FlowInfo paramFlowInfo);
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo addPotentialInitializationsFrom(org.eclipse.jdt.internal.compiler.flow.FlowInfo paramFlowInfo);
  
  public org.eclipse.jdt.internal.compiler.flow.FlowInfo asNegatedCondition() {
    return this;
  }
  
  public static org.eclipse.jdt.internal.compiler.flow.FlowInfo conditional(org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenTrue, org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenFalse) {
    if (initsWhenTrue == initsWhenFalse)
      return initsWhenTrue; 
    return (org.eclipse.jdt.internal.compiler.flow.FlowInfo)new ConditionalFlowInfo(initsWhenTrue, initsWhenFalse);
  }
  
  public boolean cannotBeDefinitelyNullOrNonNull(LocalVariableBinding local) {
    return !(!isPotentiallyUnknown(local) && (
      !isPotentiallyNonNull(local) || !isPotentiallyNull(local)));
  }
  
  public boolean cannotBeNull(LocalVariableBinding local) {
    return !(!isDefinitelyNonNull(local) && !isProtectedNonNull(local));
  }
  
  public boolean canOnlyBeNull(LocalVariableBinding local) {
    return !(!isDefinitelyNull(local) && !isProtectedNull(local));
  }
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo copy();
  
  public static UnconditionalFlowInfo initial(int maxFieldCount) {
    UnconditionalFlowInfo info = new UnconditionalFlowInfo();
    info.maxFieldCount = maxFieldCount;
    return info;
  }
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenFalse();
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenTrue();
  
  public abstract boolean isDefinitelyAssigned(FieldBinding paramFieldBinding);
  
  public abstract boolean isDefinitelyAssigned(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isDefinitelyNonNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isDefinitelyNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isDefinitelyUnknown(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean hasNullInfoFor(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isPotentiallyAssigned(FieldBinding paramFieldBinding);
  
  public abstract boolean isPotentiallyAssigned(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isPotentiallyNonNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isPotentiallyNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isPotentiallyUnknown(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isProtectedNonNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract boolean isProtectedNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsComparedEqualToNonNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsComparedEqualToNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsDefinitelyAssigned(FieldBinding paramFieldBinding);
  
  public abstract void markAsDefinitelyNonNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsDefinitelyNull(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void resetNullInfo(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markPotentiallyUnknownBit(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markPotentiallyNullBit(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markPotentiallyNonNullBit(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsDefinitelyAssigned(LocalVariableBinding paramLocalVariableBinding);
  
  public abstract void markAsDefinitelyUnknown(LocalVariableBinding paramLocalVariableBinding);
  
  public void markNullStatus(LocalVariableBinding local, int nullStatus) {
    switch (nullStatus) {
      case 1:
        markAsDefinitelyUnknown(local);
        return;
      case 2:
        markAsDefinitelyNull(local);
        return;
      case 4:
        markAsDefinitelyNonNull(local);
        return;
    } 
    resetNullInfo(local);
    if ((nullStatus & 0x8) != 0)
      markPotentiallyUnknownBit(local); 
    if ((nullStatus & 0x10) != 0)
      markPotentiallyNullBit(local); 
    if ((nullStatus & 0x20) != 0)
      markPotentiallyNonNullBit(local); 
    if ((nullStatus & 0x38) == 0)
      markAsDefinitelyUnknown(local); 
  }
  
  public int nullStatus(LocalVariableBinding local) {
    if (isDefinitelyUnknown(local))
      return 1; 
    if (isDefinitelyNull(local))
      return 2; 
    if (isDefinitelyNonNull(local))
      return 4; 
    int status = 0;
    if (isPotentiallyUnknown(local))
      status |= 0x8; 
    if (isPotentiallyNull(local))
      status |= 0x10; 
    if (isPotentiallyNonNull(local))
      status |= 0x20; 
    if (status > 0)
      return status; 
    return 1;
  }
  
  public static int mergeNullStatus(int nullStatus1, int nullStatus2) {
    boolean canBeNull = false;
    boolean canBeNonNull = false;
    switch (nullStatus1) {
      case 16:
        canBeNonNull = true;
      case 2:
        canBeNull = true;
        break;
      case 32:
        canBeNull = true;
      case 4:
        canBeNonNull = true;
        break;
    } 
    switch (nullStatus2) {
      case 16:
        canBeNonNull = true;
      case 2:
        canBeNull = true;
        break;
      case 32:
        canBeNull = true;
      case 4:
        canBeNonNull = true;
        break;
    } 
    if (canBeNull) {
      if (canBeNonNull)
        return 16; 
      return 2;
    } 
    if (canBeNonNull)
      return 4; 
    return 1;
  }
  
  public static UnconditionalFlowInfo mergedOptimizedBranches(org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenTrue, boolean isOptimizedTrue, org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenFalse, boolean isOptimizedFalse, boolean allowFakeDeadBranch) {
    UnconditionalFlowInfo mergedInfo;
    if (isOptimizedTrue) {
      if (initsWhenTrue == DEAD_END && allowFakeDeadBranch) {
        mergedInfo = initsWhenFalse.setReachMode(1)
          .unconditionalInits();
      } else {
        mergedInfo = 
          initsWhenTrue.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenFalse
            .nullInfoLessUnconditionalCopy())
          .unconditionalInits();
      } 
    } else if (isOptimizedFalse) {
      if (initsWhenFalse == DEAD_END && allowFakeDeadBranch) {
        mergedInfo = initsWhenTrue.setReachMode(1)
          .unconditionalInits();
      } else {
        mergedInfo = 
          initsWhenFalse.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenTrue
            .nullInfoLessUnconditionalCopy())
          .unconditionalInits();
      } 
    } else {
      mergedInfo = initsWhenTrue
        .mergedWith(initsWhenFalse.unconditionalInits());
    } 
    return mergedInfo;
  }
  
  public static UnconditionalFlowInfo mergedOptimizedBranchesIfElse(org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenTrue, boolean isOptimizedTrue, org.eclipse.jdt.internal.compiler.flow.FlowInfo initsWhenFalse, boolean isOptimizedFalse, boolean allowFakeDeadBranch, org.eclipse.jdt.internal.compiler.flow.FlowInfo flowInfo, IfStatement ifStatement, boolean reportDeadCodeInKnownPattern) {
    UnconditionalFlowInfo mergedInfo;
    if (isOptimizedTrue) {
      if (initsWhenTrue == DEAD_END && allowFakeDeadBranch) {
        if (!reportDeadCodeInKnownPattern) {
          if (ifStatement.elseStatement == null) {
            mergedInfo = flowInfo.unconditionalInits();
          } else {
            mergedInfo = initsWhenFalse.unconditionalInits();
            if (initsWhenFalse != DEAD_END)
              mergedInfo.setReachMode(flowInfo.reachMode()); 
          } 
        } else {
          mergedInfo = initsWhenFalse.setReachMode(1)
            .unconditionalInits();
        } 
      } else {
        mergedInfo = 
          initsWhenTrue.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenFalse
            .nullInfoLessUnconditionalCopy())
          .unconditionalInits();
      } 
    } else if (isOptimizedFalse) {
      if (initsWhenFalse == DEAD_END && allowFakeDeadBranch) {
        if (!reportDeadCodeInKnownPattern) {
          if (ifStatement.thenStatement == null) {
            mergedInfo = flowInfo.unconditionalInits();
          } else {
            mergedInfo = initsWhenTrue.unconditionalInits();
            if (initsWhenTrue != DEAD_END)
              mergedInfo.setReachMode(flowInfo.reachMode()); 
          } 
        } else {
          mergedInfo = initsWhenTrue.setReachMode(1)
            .unconditionalInits();
        } 
      } else {
        mergedInfo = 
          initsWhenFalse.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenTrue
            .nullInfoLessUnconditionalCopy())
          .unconditionalInits();
      } 
    } else if ((flowInfo.tagBits & 0x3) == 0 && (
      ifStatement.bits & 0x80) != 0 && 
      initsWhenTrue != DEAD_END && 
      initsWhenFalse != DEAD_END) {
      mergedInfo = 
        initsWhenTrue.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenFalse
          .nullInfoLessUnconditionalCopy())
        .unconditionalInits();
      mergedInfo.mergeDefiniteInitsWith(initsWhenFalse.unconditionalCopy());
      if ((mergedInfo.tagBits & 0x1) != 0 && (initsWhenFalse.tagBits & 0x3) == 2) {
        mergedInfo.tagBits &= 0xFFFFFFFE;
        mergedInfo.tagBits |= 0x2;
      } 
    } else if ((flowInfo.tagBits & 0x3) == 0 && (
      ifStatement.bits & 0x100) != 0 && initsWhenTrue != DEAD_END && 
      initsWhenFalse != DEAD_END) {
      mergedInfo = 
        initsWhenFalse.addPotentialInitializationsFrom((org.eclipse.jdt.internal.compiler.flow.FlowInfo)initsWhenTrue
          .nullInfoLessUnconditionalCopy())
        .unconditionalInits();
      mergedInfo.mergeDefiniteInitsWith(initsWhenTrue.unconditionalCopy());
      if ((mergedInfo.tagBits & 0x1) != 0 && (initsWhenTrue.tagBits & 0x3) == 2) {
        mergedInfo.tagBits &= 0xFFFFFFFE;
        mergedInfo.tagBits |= 0x2;
      } 
    } else {
      mergedInfo = initsWhenTrue
        .mergedWith(initsWhenFalse.unconditionalInits());
    } 
    return mergedInfo;
  }
  
  public int reachMode() {
    return this.tagBits & 0x3;
  }
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo safeInitsWhenTrue();
  
  public abstract org.eclipse.jdt.internal.compiler.flow.FlowInfo setReachMode(int paramInt);
  
  public abstract UnconditionalFlowInfo mergedWith(UnconditionalFlowInfo paramUnconditionalFlowInfo);
  
  public abstract UnconditionalFlowInfo mergeDefiniteInitsWith(UnconditionalFlowInfo paramUnconditionalFlowInfo);
  
  public abstract UnconditionalFlowInfo nullInfoLessUnconditionalCopy();
  
  public String toString() {
    if (this == DEAD_END)
      return "FlowInfo.DEAD_END"; 
    return super.toString();
  }
  
  public abstract UnconditionalFlowInfo unconditionalCopy();
  
  public abstract UnconditionalFlowInfo unconditionalFieldLessCopy();
  
  public abstract UnconditionalFlowInfo unconditionalInits();
  
  public abstract UnconditionalFlowInfo unconditionalInitsWithoutSideEffect();
  
  public abstract void resetAssignmentInfo(LocalVariableBinding paramLocalVariableBinding);
  
  public static int tagBitsToNullStatus(long tagBits) {
    if ((tagBits & 0x100000000000000L) != 0L)
      return 4; 
    if ((tagBits & 0x80000000000000L) != 0L)
      return 48; 
    return 1;
  }
}
